@@ -29,6 +29,8 @@ import weakref
 import gobject
 import gtk
 
+from solfege.esel import SearchView
+
 if __name__ == '__main__':
     from solfege import i18n
     i18n.setup(".", "C")
@@ -90,7 +92,7 @@ class LessonFilePreviewWidget(gtk.VBox):
                     l = gtk.Label(u"-")
                     l.set_alignment(0.0, 0.5)
                     self.g_topic_box.pack_start(l, False)
-            except (lessonfile.InfoCache.FileNotFound, 
+            except (lessonfile.InfoCache.FileNotFound,
                     lessonfile.InfoCache.FileNotLessonfile), e:
                 self.g_title.set_text(u'')
                 self.g_module.set_text(u'')
@@ -113,6 +115,19 @@ class SelectLessonFileDialog(gtk.FileChooserDialog):
         self.connect('selection-changed', pv.update)
 
 
+class SelectLessonfileBySearchDialog(gtk.Dialog):
+    def __init__(self):
+        gtk.Dialog.__init__(self, buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_ACCEPT))
+        view = SearchView(_('Search for exercises. Each exercise you click will be added to the section of the front page.'),
+            fields=['link-with-filename-tooltip', 'module'])
+        view.on_link_clicked = self.on_link_clicked
+        self.vbox.pack_start(view)
+        self.show_all()
+    def on_link_clicked(self, widget, filename):
+        self.m_filename = filename
+        self.response(gtk.RESPONSE_OK)
+
+
 def editor_of(obj):
     """
     Return the toplevel page, the one that is a Editor object.
@@ -133,7 +148,7 @@ def parent_page(obj):
             return p
         if p is None:
             return None
-    
+
 class Section(gtk.VBox):
     """
     A section consists of a heading and a list of links.
@@ -269,10 +284,22 @@ class Section(gtk.VBox):
         item = gtk.MenuItem(_("Add link to exercise"))
         item.connect('activate', self.on_add_link)
         menu.append(item)
+        item = gtk.MenuItem(_("Add link by searching for exercises"))
+        item.connect('activate', self.on_add_link_by_search)
+        menu.append(item)
         menu.show_all()
         menu.popup(None, None, None, event.button, event.time)
     def on_remove(self, btn, event):
         self.m_parent.remove_section(self)
+    def on_add_link_by_search(self, btn):
+        dlg = SelectLessonfileBySearchDialog()
+        while True:
+            ret = dlg.run()
+            if ret == gtk.RESPONSE_OK:
+                self._add_filenames([os.path.abspath(lessonfile.uri_expand(dlg.m_filename))])
+            else:
+                break
+        dlg.destroy()
     def on_add_link(self, btn):
         if editor_of(self).m_filename:
             open_dir = os.path.split(editor_of(self).m_filename)[0]
@@ -286,23 +313,25 @@ class Section(gtk.VBox):
                 break
             else:
                 assert ret == gtk.RESPONSE_OK
-                for filename in dlg.get_filenames():
-                    fn = gu.decode_filename(filename)
-                    assert os.path.isabs(fn)
-                    # If the file name is a file in a subdirectory below
-                    # lessonfile.exercises_dir in the current working directory,
-                    # then the file is a standard lesson file, and it will be
-                    # converted to a uri scheme with:
-                    fn = lessonfile.mk_uri(fn)
-                    # Small test to check that the file actually is a lesson file.
-                    try:
-                        lessonfile.infocache.get(fn, 'title')
-                    except lessonfile.infocache.FileNotLessonfile:
-                        continue
-                    self.m_model.append(fn)
-                    self.g_link_box.pack_start(self.create_linkrow(fn), False)
+                self._add_filenames(dlg.get_filenames())
                 break
         dlg.destroy()
+    def _add_filenames(self, filenames):
+        for filename in filenames:
+            fn = gu.decode_filename(filename)
+            assert os.path.isabs(fn)
+            # If the file name is a file in a subdirectory below
+            # lessonfile.exercises_dir in the current working directory,
+            # then the file is a standard lesson file, and it will be
+            # converted to a uri scheme with:
+            fn = lessonfile.mk_uri(fn)
+            # Small test to check that the file actually is a lesson file.
+            try:
+                lessonfile.infocache.get(fn, 'title')
+            except lessonfile.infocache.FileNotLessonfile:
+                continue
+            self.m_model.append(fn)
+            self.g_link_box.pack_start(self.create_linkrow(fn), False)
     def on_add_link_to_new_page(self, menuitem):
         page = pd.Page(_("Untitled%s") % "", [pd.Column()])
         self.m_model.append(page)
@@ -603,7 +632,7 @@ class Clipboard(list):
         list.append(self, obj)
         self.update_buttons()
     def register_paste_button(self, button, accepts_types):
-        button.set_sensitive(bool(self) and isinstance(self[-1], types))
+        button.set_sensitive(bool(self) and isinstance(self[-1], accepts_types))
         self.m_paste_buttons.append((button, accepts_types))
     def update_buttons(self):
         for button, types in self.m_paste_buttons:
@@ -734,7 +763,7 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         assert self.m_model == None
         self.m_model = pd.load_tree(filename, C_locale=True)
         self.m_filename = filename
-        # 
+        #
         if not os.path.isabs(filename):
             if not os.access(filename, os.W_OK):
                 m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,
@@ -771,7 +800,7 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         self.set_not_modified()
         # We do test for solfege.win since it is not available during testing
         if getattr(solfege, 'win', None):
-            solfege.win.on_frontpage_changed()
+            solfege.win.load_frontpage()
     def on_show_help(self, *w):
         return
     def get_save_as_dialog(self):
