@@ -3,6 +3,7 @@
 from __future__ import absolute_import
 
 import copy
+import os
 import re
 
 import gtk
@@ -31,7 +32,10 @@ class Controller(gtk.HBox):
         gtk.HBox.__init__(self)
         self.g_rwidget = rwidget
         for k in (1, 2, 4, 8, 16, 32):
-            b = gtk.Button(unicode(k))
+            im = gtk.Image()
+            im.set_from_file(os.path.join("graphics", "note-%i.svg"% k))
+            b = gtk.Button()
+            b.add(im)
             self.pack_start(b, False)
             def f(widget, i):
                 self.g_rwidget.on_add_item(elems.Note(
@@ -40,20 +44,32 @@ class Controller(gtk.HBox):
                 self.g_rwidget.grab_focus()
             b.connect('clicked', f, k)
         for k in (1, 2, 4, 8, 16, 32):
-            b = gtk.Button(u"r%i" % k)
+            im = gtk.Image()
+            im.set_from_file(os.path.join("graphics", "rest-%i.svg" % k))
+            b = gtk.Button()
+            b.add(im)
             self.pack_start(b, False)
             def f(widget, i):
                 self.g_rwidget.on_add_item(elems.Rest(
                     Duration(i, 0)))
                 self.g_rwidget.grab_focus()
             b.connect('clicked', f, k)
-        b = gtk.Button("+.")
+        im = gtk.Image()
+        im.set_from_file(os.path.join("graphics", "add-dot.svg"))
+        b = gtk.Button()
+        b.add(im)
         b.connect('clicked', self.on_toggle_dots, 1)
         self.pack_start(b, False)
-        b = gtk.Button("-.")
+        im = gtk.Image()
+        im.set_from_file(os.path.join("graphics", "remove-dot.svg"))
+        b = gtk.Button()
+        b.add(im)
         b.connect('clicked', self.on_toggle_dots, -1)
         self.pack_start(b, False)
-        b = gtk.Button("~")
+        im = gtk.Image()
+        im.set_from_file(os.path.join("graphics", "tie.svg"))
+        b = gtk.Button()
+        b.add(im)
         b.connect('clicked', self.on_toggle_tie)
         self.pack_start(b, False)
         im = gtk.Image()
@@ -62,7 +78,16 @@ class Controller(gtk.HBox):
         b.add(im)
         b.connect('clicked', self.ctrl_on_delete)
         self.pack_start(b, False)
+        self.g_mode = gtk.ToggleButton(_i("insert-overwrite|INSRT"))
+        self.g_mode.connect('clicked', self.ctrl_on_ins)
+        self.g_rwidget.m_ins_mode = not self.g_mode.get_active()
+        self.pack_start(self.g_mode, False)
         self.show_all()
+    def ctrl_on_ins(self, button):
+        self.g_rwidget.m_ins_mode = not self.g_mode.get_active()
+        self.g_mode.set_label({False: _i("insert-overwrite|INSRT"),
+            True: _i("insert-overwrite|OVER")}[button.get_active()])
+        self.g_rwidget.grab_focus()
     def ctrl_on_delete(self, button):
         self.g_rwidget.on_delete()
         self.g_rwidget.grab_focus()
@@ -91,12 +116,12 @@ class RhythmWidget(MusicDisplayer):
         self.g_d.connect("expose_event", self.on_draw_cursor)
         self.add_events(gtk.gdk.KEY_RELEASE_MASK)
         self.add_events(gtk.gdk.BUTTON_PRESS_MASK)
-        self.connect("key-release-event", self.on_keypress) 
+        self.connect("key-release-event", self.on_keypress)
         self.set_flags(gtk.CAN_FOCUS)
         def f(*w):
             self.grab_focus()
         self.connect("button-press-event", f)
-        self.fill_score(elems.TimeSignature(4, 4), elems.Skip(RhythmWidget.skipdur), 4)
+        self.m_cursor = None
     def get_cursor_timepos(self):
         """
         Return the timepos the cursor has. Return None if the
@@ -105,6 +130,8 @@ class RhythmWidget(MusicDisplayer):
         """
         if self.m_cursor is None:
             return
+        if not self.m_score.m_staffs:
+            return None
         timeposes = self.m_score.m_staffs[0].get_timeposes()
         if timeposes:
             return self.m_score.m_staffs[0].get_timeposes()[self.m_cursor]
@@ -117,7 +144,7 @@ class RhythmWidget(MusicDisplayer):
             self.m_cursor += 1
     def on_keypress(self, window, event):
         if not self.m_editable:
-            return 
+            return
         if event.keyval in (gtk.keysyms.Right, gtk.keysyms.KP_Right):
             self.cursor_next()
         elif event.keyval in (gtk.keysyms.Left, gtk.keysyms.KP_Left):
@@ -131,7 +158,7 @@ class RhythmWidget(MusicDisplayer):
             if self.m_score.voice11.tie_timepos(timepos):
                 self.score_updated()
         elif self.m_score.voice11.m_tdict[timepos]['elem'][0].m_tieinfo in ('start', 'go'):
-            if self.m_score.voice11.untie_timepos(timepos):
+            if self.m_score.voice11.untie_next(timepos):
                 self.score_updated()
     def on_delete(self):
         timepos = self.get_cursor_timepos()
@@ -145,15 +172,18 @@ class RhythmWidget(MusicDisplayer):
         """
         timepos = self.get_cursor_timepos()
         elem = self.m_score.voice11.m_tdict[timepos]['elem'][0]
+        if isinstance(elem, elems.Skip):
+            return False
         if elem.m_duration.m_dots + delta < 0:
             return False
         new_elem = copy.deepcopy(self.m_score.voice11.m_tdict[timepos]['elem'][0])
         new_elem.m_duration.m_dots += delta
-        if self.m_score.voice11.try_set_elem(new_elem, timepos):
+        if self.m_score.voice11.try_set_elem(new_elem, timepos, False):
             self.score_updated()
         return True
     def on_add_item(self, item):
-        if self.m_score.voice11.try_set_elem(item, self.get_cursor_timepos()):
+        if self.m_score.voice11.try_set_elem(item, self.get_cursor_timepos(),
+                self.m_ins_mode):
             self.cursor_next()
             self.score_updated()
     def fill_score(self, timesig, skip, count):
@@ -172,26 +202,26 @@ class RhythmWidget(MusicDisplayer):
         """
         Redraw the staff. This should be called whenever m_score is updated.
         """
-        if not self.m_score.m_staffs:
-            return
         self.m_scorecontext = engravers.ScoreContext(self.m_score)
         self.m_engravers = self.m_scorecontext.m_contexts
         self._display()
-        timeposes = self.m_score.m_staffs[0].get_timeposes()
-        if self.m_cursor > len(timeposes) - 1:
-            self.m_cursor = len(timeposes) - 1
+        if self.m_score.m_staffs:
+            timeposes = self.m_score.m_staffs[0].get_timeposes()
+            if self.m_cursor > len(timeposes) - 1:
+                self.m_cursor = len(timeposes) - 1
+        else:
+            self.m_cursor = None
     def on_draw_cursor(self, darea, event):
-        if not self.m_scorecontext.m_contexts[0].m_engravers:
-            return
-        engraver = None
         timepos = self.get_cursor_timepos()
         if not timepos:
             return
+        engraver = None
         for e in self.m_scorecontext.m_contexts[0].m_engravers[timepos]['elem']:
             if isinstance(e, (engravers.NoteheadEngraver,
                               engravers.RestEngraver,
                               engravers.SkipEngraver)):
                 engraver = e
+                break
         if not engraver:
             return
         red = self.window.get_colormap().alloc_color('#FF0000', True, True)
@@ -207,6 +237,11 @@ class TestWin(gtk.Window):
         self.add(vbox)
         self.set_default_size(600, 400)
         self.w = RhythmWidget()
+        s = elems.Score()
+        s.add_staff(staff_class=elems.RhythmStaff)
+        for x in range(8):
+            s.voice11.append(elems.Skip.new_from_string("4"))
+        self.w.set_score(s)
         vbox.pack_start(self.w)
         #
         c = Controller(self.w)
